@@ -278,17 +278,20 @@ inline void chop_prefix(std::string& str, const std::string& prefix)
 inline std::string base_class_name(const std::string& name)
 {
   std::string result = name;
-  
+
   // if the name begins with "struct ", chop it off
   chop_prefix(result, "struct ");
-  
+
   // if the name begins with "class ", chop it off
   chop_prefix(result, "class ");
 
-  // chop everything including and after first "<"
-  return result.replace(result.find_first_of("<"),
-                        result.size(),
-                        "");
+  const std::size_t first_lt = result.find_first_of("<");
+
+  if (first_lt < result.size())
+      // chop everything including and after first "<"
+      return result.replace(first_lt, result.size(), "");
+  else
+      return result;
 }
 
 enum TestStatus { Pass = 0, Failure = 1, KnownFailure = 2, Error = 3, UnknownException = 4};
@@ -307,7 +310,7 @@ class UnitTest {
         virtual ~UnitTest() {}
         virtual void run() {}
 
-        bool operator<(const UnitTest& u) const 
+        bool operator<(const UnitTest& u) const
         {
             return name < u.name;
         }
@@ -335,7 +338,7 @@ public:
 
   void register_test(UnitTest * test);
   virtual bool run_tests(const ArgumentSet& args, const ArgumentMap& kwargs);
-  void list_tests(void); 
+  void list_tests(void);
 
   static UnitTestDriver &s_driver();
 };
@@ -351,11 +354,21 @@ class TEST##UnitTest : public UnitTest {                         \
 };                                                               \
 TEST##UnitTest TEST##Instance
 
+#define DECLARE_UNITTEST_WITH_NAME(TEST, NAME)                   \
+class NAME##UnitTest : public UnitTest {                         \
+    public:                                                      \
+    NAME##UnitTest() : UnitTest(#NAME) {}                        \
+    void run(){                                                  \
+            TEST();                                              \
+    }                                                            \
+};                                                               \
+NAME##UnitTest NAME##Instance
+
 // Macro to create host and device versions of a
 // unit test for a bunch of data types
 #define DECLARE_VECTOR_UNITTEST(VTEST)                          \
 void VTEST##Host(void) {                                        \
-    VTEST< thrust::host_vector<char> >();                       \
+    VTEST< thrust::host_vector<signed char> >();                \
     VTEST< thrust::host_vector<short> >();                      \
     VTEST< thrust::host_vector<int> >();                        \
     VTEST< thrust::host_vector<float> >();                      \
@@ -366,7 +379,7 @@ void VTEST##Host(void) {                                        \
             thrust::host_memory_resource> > >();                \
 }                                                               \
 void VTEST##Device(void) {                                      \
-    VTEST< thrust::device_vector<char> >();                     \
+    VTEST< thrust::device_vector<signed char> >();              \
     VTEST< thrust::device_vector<short> >();                    \
     VTEST< thrust::device_vector<int> >();                      \
     VTEST< thrust::device_vector<float> >();                    \
@@ -388,12 +401,12 @@ DECLARE_UNITTEST(VTEST##Device);
 // Same as above, but only for integral types
 #define DECLARE_INTEGRAL_VECTOR_UNITTEST(VTEST)                 \
 void VTEST##Host(void) {                                        \
-    VTEST< thrust::host_vector<char> >();                       \
+    VTEST< thrust::host_vector<signed char> >();                \
     VTEST< thrust::host_vector<short> >();                      \
     VTEST< thrust::host_vector<int> >();                        \
 }                                                               \
 void VTEST##Device(void) {                                      \
-    VTEST< thrust::device_vector<char> >();                     \
+    VTEST< thrust::device_vector<signed char> >();              \
     VTEST< thrust::device_vector<short> >();                    \
     VTEST< thrust::device_vector<int> >();                      \
 }                                                               \
@@ -407,7 +420,7 @@ class TEST##UnitTest : public UnitTest {                         \
     TEST##UnitTest() : UnitTest(#TEST) {}                        \
     void run()                                                   \
     {                                                            \
-        TEST<char>();                                            \
+        TEST<signed char>();                                     \
         TEST<unsigned char>();                                   \
         TEST<short>();                                           \
         TEST<unsigned short>();                                  \
@@ -428,7 +441,7 @@ class TEST##UnitTest : public UnitTest {                         \
         std::vector<size_t> sizes = get_test_sizes();            \
         for(size_t i = 0; i != sizes.size(); ++i)                \
         {                                                        \
-            TEST<char>(sizes[i]);                                \
+            TEST<signed char>(sizes[i]);                         \
             TEST<unsigned char>(sizes[i]);                       \
             TEST<short>(sizes[i]);                               \
             TEST<unsigned short>(sizes[i]);                      \
@@ -450,7 +463,7 @@ class TEST##UnitTest : public UnitTest {                         \
         std::vector<size_t> sizes = get_test_sizes();            \
         for(size_t i = 0; i != sizes.size(); ++i)                \
         {                                                        \
-            TEST<char>(sizes[i]);                                \
+            TEST<signed char>(sizes[i]);                         \
             TEST<unsigned char>(sizes[i]);                       \
             TEST<short>(sizes[i]);                               \
             TEST<unsigned short>(sizes[i]);                      \
@@ -514,7 +527,7 @@ template<template <typename> class TestName, typename TypeList>
     {
         std::vector<size_t> sizes = get_test_sizes();
         for(size_t i = 0; i != sizes.size(); ++i)
-        {                                                 
+        {
             // get the first type in the list
             typedef typename unittest::get_type<TypeList,0>::type first_type;
 
@@ -522,7 +535,7 @@ template<template <typename> class TestName, typename TypeList>
 
             // loop over the types
             loop(sizes[i]);
-        }                                                 
+        }
     }
 }; // end VariableUnitTest
 
@@ -534,7 +547,7 @@ template<template <typename> class TestName,
     : public UnitTest
 {
   VectorUnitTest()
-    : UnitTest((base_class_name(unittest::type_name<TestName< Vector<int, Alloc<int> > > >()) + "<" + 
+    : UnitTest((base_class_name(unittest::type_name<TestName< Vector<int, Alloc<int> > > >()) + "<" +
                 base_class_name(unittest::type_name<Vector<int, Alloc<int> > >()) + ">").c_str())
   { }
 
