@@ -33,7 +33,7 @@
 #include <thrust/detail/cstdint.h>
 #include <thrust/detail/temporary_array.h>
 #include <thrust/system/cuda/detail/util.h>
-#include <thrust/system/cuda/detail/cub/device/device_select.cuh>
+#include <cub/device/device_select.cuh>
 #include <thrust/system/cuda/detail/core/agent_launcher.h>
 #include <thrust/system/cuda/detail/core/util.h>
 #include <thrust/system/cuda/detail/par_to_seq.h>
@@ -41,7 +41,8 @@
 #include <thrust/distance.h>
 #include <thrust/detail/alignment.h>
 
-THRUST_BEGIN_NS
+namespace thrust
+{
 // XXX declare generic copy_if interface
 // to avoid circulular dependency from thrust/copy.h
 template <typename DerivedPolicy, typename InputIterator, typename OutputIterator, typename Predicate>
@@ -69,7 +70,6 @@ namespace __copy_if {
 
   template <int                     _BLOCK_THREADS,
             int                     _ITEMS_PER_THREAD = 1,
-            int                     _MIN_BLOCKS       = 1,
             cub::BlockLoadAlgorithm _LOAD_ALGORITHM   = cub::BLOCK_LOAD_DIRECT,
             cub::CacheLoadModifier  _LOAD_MODIFIER    = cub::LOAD_LDG,
             cub::BlockScanAlgorithm _SCAN_ALGORITHM   = cub::BLOCK_SCAN_WARP_SCANS>
@@ -79,7 +79,6 @@ namespace __copy_if {
     {
       BLOCK_THREADS      = _BLOCK_THREADS,
       ITEMS_PER_THREAD   = _ITEMS_PER_THREAD,
-      MIN_BLOCKS         = _MIN_BLOCKS,
       ITEMS_PER_TILE     = _BLOCK_THREADS * _ITEMS_PER_THREAD,
     };
     static const cub::BlockLoadAlgorithm LOAD_ALGORITHM = _LOAD_ALGORITHM;
@@ -89,7 +88,7 @@ namespace __copy_if {
 
   template<class, class>
   struct Tuning;
-  
+
   template<class T>
   struct Tuning<sm52, T>
   {
@@ -103,13 +102,12 @@ namespace __copy_if {
 
     typedef PtxPolicy<128,
                       ITEMS_PER_THREAD,
-                      1,
                       cub::BLOCK_LOAD_WARP_TRANSPOSE,
                       cub::LOAD_LDG,
                       cub::BLOCK_SCAN_WARP_SCANS>
         type;
   };    // Tuning<350>
-  
+
 
   template<class T>
   struct Tuning<sm35, T>
@@ -124,13 +122,12 @@ namespace __copy_if {
 
     typedef PtxPolicy<128,
                       ITEMS_PER_THREAD,
-                      1,
                       cub::BLOCK_LOAD_WARP_TRANSPOSE,
                       cub::LOAD_LDG,
                       cub::BLOCK_SCAN_WARP_SCANS>
         type;
   };    // Tuning<350>
-  
+
   template<class T>
   struct Tuning<sm30, T>
   {
@@ -144,13 +141,12 @@ namespace __copy_if {
 
     typedef PtxPolicy<128,
                       ITEMS_PER_THREAD,
-                      1,
                       cub::BLOCK_LOAD_WARP_TRANSPOSE,
                       cub::LOAD_DEFAULT,
                       cub::BLOCK_SCAN_WARP_SCANS>
         type;
   };    // Tuning<300>
-  
+
   struct no_stencil_tag_    {};
   typedef no_stencil_tag_* no_stencil_tag;
   template <class ItemsIt,
@@ -206,7 +202,7 @@ namespace __copy_if {
         core::uninitialized_array<item_type, PtxPlan::ITEMS_PER_TILE> raw_exchange;
       };    // union TempStorage
     };    // struct PtxPlan
-    
+
     typedef typename core::specialize_plan_msvc10_war<PtxPlan>::type::type ptx_plan;
 
     typedef typename ptx_plan::ItemsLoadIt        ItemsLoadIt;
@@ -224,7 +220,7 @@ namespace __copy_if {
       ITEMS_PER_THREAD = ptx_plan::ITEMS_PER_THREAD,
       ITEMS_PER_TILE   = ptx_plan::ITEMS_PER_TILE
     };
-    
+
     struct impl
     {
       //---------------------------------------------------------------------
@@ -238,7 +234,7 @@ namespace __copy_if {
       OutputIt       output_it;
       Predicate      predicate;
       Size           num_items;
-      
+
       //------------------------------------------
       // scatter results to memory
       //------------------------------------------
@@ -272,7 +268,7 @@ namespace __copy_if {
           output_it[num_selections_prefix + item] = storage.raw_exchange[item];
         }
       }    // func scatter
-      
+
       //------------------------------------------
       // specialize predicate on different types
       //------------------------------------------
@@ -357,11 +353,11 @@ namespace __copy_if {
           }
         }
       }
-      
+
       //------------------------------------------
       // consume tiles
       //------------------------------------------
-      
+
       template <bool IS_LAST_TILE, bool IS_FIRST_TILE>
       Size THRUST_DEVICE_FUNCTION
       consume_tile_impl(int  num_tile_items,
@@ -501,7 +497,7 @@ namespace __copy_if {
       //---------------------------------------------------------------------
       // Constructor
       //---------------------------------------------------------------------
-      
+
       THRUST_DEVICE_FUNCTION impl(TempStorage &       storage_,
                                   ScanTileState &     tile_state_,
                                   ItemsIt             items_it,
@@ -578,7 +574,7 @@ namespace __copy_if {
     template <class Arch>
     struct PtxPlan : PtxPolicy<128> {};
     typedef core::specialize_plan<PtxPlan> ptx_plan;
-    
+
     //---------------------------------------------------------------------
     // Agent entry point
     //---------------------------------------------------------------------
@@ -648,11 +644,11 @@ namespace __copy_if {
     cudaError_t status = cudaSuccess;
     if (num_items == 0)
       return status;
-    
+
     size_t allocation_sizes[2] = {0, vshmem_size};
     status = ScanTileState::AllocationSize(static_cast<int>(num_tiles), allocation_sizes[0]);
     CUDA_CUB_RET_IF_FAIL(status);
-    
+
 
     void* allocations[2] = {NULL, NULL};
     status = cub::AliasTemporaries(d_temp_storage,
@@ -660,7 +656,7 @@ namespace __copy_if {
                                    allocations,
                                    allocation_sizes);
     CUDA_CUB_RET_IF_FAIL(status);
-    
+
 
     if (d_temp_storage == NULL)
     {
@@ -855,7 +851,7 @@ copy_if(execution_policy<Derived> &policy,
 }    // func copy_if
 
 }    // namespace cuda_cub
-THRUST_END_NS
+} // end namespace thrust
 
 #include <thrust/copy.h>
 #endif
