@@ -42,16 +42,17 @@ template<int...> class ID_impl;
 template<int... I> class Foo { ID_impl<I...> t;};
 #endif
 
-THRUST_BEGIN_NS
+namespace thrust
+{
 namespace cuda_cub {
 namespace core {
 
 
-#ifdef __CUDA_ARCH__
+#if defined(__CUDA_ARCH__) || defined(__NVCOMPILER_CUDA__)
 #if 0
   template <class Agent, class... Args>
-  void __global__ 
-  __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS,Agent::ptx_plan::MIN_BLOCKS)
+  void __global__
+  __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS)
       _kernel_agent(Args... args)
   {
     extern __shared__ char shmem[];
@@ -59,119 +60,119 @@ namespace core {
   }
 #else
   template <class Agent, class _0>
-  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS, Agent::ptx_plan::MIN_BLOCKS)
+  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS)
   _kernel_agent(_0 x0)
   {
     extern __shared__ char shmem[];
     Agent::entry(x0, shmem);
   }
   template <class Agent, class _0, class _1>
-  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS, Agent::ptx_plan::MIN_BLOCKS)
+  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS)
   _kernel_agent(_0 x0, _1 x1)
   {
     extern __shared__ char shmem[];
     Agent::entry(x0, x1, shmem);
   }
   template <class Agent, class _0, class _1, class _2>
-  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS, Agent::ptx_plan::MIN_BLOCKS)
+  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS)
   _kernel_agent(_0 x0, _1 x1, _2 x2)
   {
     extern __shared__ char shmem[];
     Agent::entry(x0, x1, x2, shmem);
   }
   template <class Agent, class _0, class _1, class _2, class _3>
-  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS, Agent::ptx_plan::MIN_BLOCKS)
+  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS)
   _kernel_agent(_0 x0, _1 x1, _2 x2, _3 x3)
   {
     extern __shared__ char shmem[];
     Agent::entry(x0, x1, x2, x3, shmem);
   }
   template <class Agent, class _0, class _1, class _2, class _3, class _4>
-  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS, Agent::ptx_plan::MIN_BLOCKS)
+  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS)
   _kernel_agent(_0 x0, _1 x1, _2 x2, _3 x3, _4 x4)
   {
     extern __shared__ char shmem[];
     Agent::entry(x0, x1, x2, x3, x4, shmem);
   }
   template <class Agent, class _0, class _1, class _2, class _3, class _4, class _5>
-  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS, Agent::ptx_plan::MIN_BLOCKS)
+  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS)
   _kernel_agent(_0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5)
   {
     extern __shared__ char shmem[];
     Agent::entry(x0, x1, x2, x3, x4, x5, shmem);
   }
   template <class Agent, class _0, class _1, class _2, class _3, class _4, class _5, class _6>
-  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS, Agent::ptx_plan::MIN_BLOCKS)
+  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS)
   _kernel_agent(_0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6)
   {
     extern __shared__ char shmem[];
     Agent::entry(x0, x1, x2, x3, x4, x5, x6, shmem);
   }
   template <class Agent, class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7>
-  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS, Agent::ptx_plan::MIN_BLOCKS)
+  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS)
   _kernel_agent(_0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7)
   {
     extern __shared__ char shmem[];
     Agent::entry(x0, x1, x2, x3, x4, x5, x6, x7, shmem);
   }
   template <class Agent, class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8>
-  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS, Agent::ptx_plan::MIN_BLOCKS)
+  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS)
   _kernel_agent(_0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8)
   {
     extern __shared__ char shmem[];
     Agent::entry(x0, x1, x2, x3, x4, x5, x6, x7, x8, shmem);
   }
   template <class Agent, class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8, class _9>
-  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS, Agent::ptx_plan::MIN_BLOCKS)
+  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS)
   _kernel_agent(_0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8, _9 x9)
   {
     extern __shared__ char shmem[];
     Agent::entry(x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, shmem);
   }
   template <class Agent, class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8, class _9, class _xA>
-  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS, Agent::ptx_plan::MIN_BLOCKS)
+  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS)
   _kernel_agent(_0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8, _9 x9, _xA xA)
   {
     extern __shared__ char shmem[];
     Agent::entry(x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, xA, shmem);
   }
   template <class Agent, class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8, class _9, class _xA, class _xB>
-  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS, Agent::ptx_plan::MIN_BLOCKS)
+  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS)
   _kernel_agent(_0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8, _9 x9, _xA xA, _xB xB)
   {
     extern __shared__ char shmem[];
     Agent::entry(x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, xA, xB, shmem);
   }
   template <class Agent, class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8, class _9, class _xA, class _xB, class _xC>
-  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS, Agent::ptx_plan::MIN_BLOCKS)
+  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS)
   _kernel_agent(_0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8, _9 x9, _xA xA, _xB xB, _xC xC)
   {
     extern __shared__ char shmem[];
     Agent::entry(x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, xA, xB, xC, shmem);
   }
   template <class Agent, class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8, class _9, class _xA, class _xB, class _xC, class _xD>
-  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS, Agent::ptx_plan::MIN_BLOCKS)
+  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS)
   _kernel_agent(_0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8, _9 x9, _xA xA, _xB xB, _xC xC, _xD xD)
   {
     extern __shared__ char shmem[];
     Agent::entry(x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, xA, xB, xC, xD, shmem);
   }
   template <class Agent, class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8, class _9, class _xA, class _xB, class _xC, class _xD, class _xE>
-  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS, Agent::ptx_plan::MIN_BLOCKS)
+  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS)
   _kernel_agent(_0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8, _9 x9, _xA xA, _xB xB, _xC xC, _xD xD, _xE xE)
   {
     extern __shared__ char shmem[];
     Agent::entry(x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, xA, xB, xC, xD, xE, shmem);
   }
 #endif
-  
+
   ////////////////////////////////////////////////////////////
 
 
 #if 0
   template <class Agent, class... Args>
-  void __global__ 
-  __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS,Agent::ptx_plan::MIN_BLOCKS)
+  void __global__
+  __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS)
       _kernel_agent_vshmem(char* vshmem, Args... args)
   {
     extern __shared__ char shmem[];
@@ -180,7 +181,7 @@ namespace core {
   }
 #else
   template <class Agent, class _0>
-  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS, Agent::ptx_plan::MIN_BLOCKS)
+  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS)
   _kernel_agent_vshmem(char* vshmem, _0 x0)
   {
     extern __shared__ char shmem[];
@@ -188,7 +189,7 @@ namespace core {
     Agent::entry(x0, vshmem);
   }
   template <class Agent, class _0, class _1>
-  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS, Agent::ptx_plan::MIN_BLOCKS)
+  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS)
   _kernel_agent_vshmem(char* vshmem, _0 x0, _1 x1)
   {
     extern __shared__ char shmem[];
@@ -196,7 +197,7 @@ namespace core {
     Agent::entry(x0, x1, vshmem);
   }
   template <class Agent, class _0, class _1, class _2>
-  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS, Agent::ptx_plan::MIN_BLOCKS)
+  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS)
   _kernel_agent_vshmem(char* vshmem, _0 x0, _1 x1, _2 x2)
   {
     extern __shared__ char shmem[];
@@ -204,7 +205,7 @@ namespace core {
     Agent::entry(x0, x1, x2, vshmem);
   }
   template <class Agent, class _0, class _1, class _2, class _3>
-  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS, Agent::ptx_plan::MIN_BLOCKS)
+  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS)
   _kernel_agent_vshmem(char* vshmem, _0 x0, _1 x1, _2 x2, _3 x3)
   {
     extern __shared__ char shmem[];
@@ -212,7 +213,7 @@ namespace core {
     Agent::entry(x0, x1, x2, x3, vshmem);
   }
   template <class Agent, class _0, class _1, class _2, class _3, class _4>
-  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS, Agent::ptx_plan::MIN_BLOCKS)
+  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS)
   _kernel_agent_vshmem(char* vshmem, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4)
   {
     extern __shared__ char shmem[];
@@ -220,7 +221,7 @@ namespace core {
     Agent::entry(x0, x1, x2, x3, x4, vshmem);
   }
   template <class Agent, class _0, class _1, class _2, class _3, class _4, class _5>
-  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS, Agent::ptx_plan::MIN_BLOCKS)
+  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS)
   _kernel_agent_vshmem(char* vshmem, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5)
   {
     extern __shared__ char shmem[];
@@ -228,7 +229,7 @@ namespace core {
     Agent::entry(x0, x1, x2, x3, x4, x5, vshmem);
   }
   template <class Agent, class _0, class _1, class _2, class _3, class _4, class _5, class _6>
-  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS, Agent::ptx_plan::MIN_BLOCKS)
+  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS)
   _kernel_agent_vshmem(char* vshmem, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6)
   {
     extern __shared__ char shmem[];
@@ -236,7 +237,7 @@ namespace core {
     Agent::entry(x0, x1, x2, x3, x4, x5, x6, vshmem);
   }
   template <class Agent, class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7>
-  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS, Agent::ptx_plan::MIN_BLOCKS)
+  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS)
   _kernel_agent_vshmem(char* vshmem, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7)
   {
     extern __shared__ char shmem[];
@@ -244,7 +245,7 @@ namespace core {
     Agent::entry(x0, x1, x2, x3, x4, x5, x6, x7, vshmem);
   }
   template <class Agent, class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8>
-  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS, Agent::ptx_plan::MIN_BLOCKS)
+  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS)
   _kernel_agent_vshmem(char* vshmem, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8)
   {
     extern __shared__ char shmem[];
@@ -252,7 +253,7 @@ namespace core {
     Agent::entry(x0, x1, x2, x3, x4, x5, x6, x7, x8, vshmem);
   }
   template <class Agent, class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8, class _9>
-  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS, Agent::ptx_plan::MIN_BLOCKS)
+  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS)
   _kernel_agent_vshmem(char* vshmem, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8, _9 x9)
   {
     extern __shared__ char shmem[];
@@ -260,7 +261,7 @@ namespace core {
     Agent::entry(x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, vshmem);
   }
   template <class Agent, class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8, class _9, class _xA>
-  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS, Agent::ptx_plan::MIN_BLOCKS)
+  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS)
   _kernel_agent_vshmem(char* vshmem, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8, _9 x9, _xA xA)
   {
     extern __shared__ char shmem[];
@@ -268,7 +269,7 @@ namespace core {
     Agent::entry(x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, xA, vshmem);
   }
   template <class Agent, class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8, class _9, class _xA, class _xB>
-  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS, Agent::ptx_plan::MIN_BLOCKS)
+  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS)
   _kernel_agent_vshmem(char* vshmem, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8, _9 x9, _xA xA, _xB xB)
   {
     extern __shared__ char shmem[];
@@ -276,7 +277,7 @@ namespace core {
     Agent::entry(x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, xA, xB, vshmem);
   }
   template <class Agent, class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8, class _9, class _xA, class _xB, class _xC>
-  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS, Agent::ptx_plan::MIN_BLOCKS)
+  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS)
   _kernel_agent_vshmem(char* vshmem, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8, _9 x9, _xA xA, _xB xB, _xC xC)
   {
     extern __shared__ char shmem[];
@@ -284,7 +285,7 @@ namespace core {
     Agent::entry(x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, xA, xB, xC, vshmem);
   }
   template <class Agent, class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8, class _9, class _xA, class _xB, class _xC, class _xD>
-  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS, Agent::ptx_plan::MIN_BLOCKS)
+  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS)
   _kernel_agent_vshmem(char* vshmem, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8, _9 x9, _xA xA, _xB xB, _xC xC, _xD xD)
   {
     extern __shared__ char shmem[];
@@ -292,7 +293,7 @@ namespace core {
     Agent::entry(x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, xA, xB, xC, xD, vshmem);
   }
   template <class Agent, class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8, class _9, class _xA, class _xB, class _xC, class _xD, class _xE>
-  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS, Agent::ptx_plan::MIN_BLOCKS)
+  void __global__ __launch_bounds__(Agent::ptx_plan::BLOCK_THREADS)
   _kernel_agent_vshmem(char* vshmem, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8, _9 x9, _xA xA, _xB xB, _xC xC, _xD xD, _xE xE)
   {
     extern __shared__ char shmem[];
@@ -397,7 +398,7 @@ namespace core {
                                   MAX_SHMEM_PER_BLOCK> shm1;
 
     template <class Size>
-    CUB_RUNTIME_FUNCTION
+    THRUST_RUNTIME_FUNCTION
     AgentLauncher(AgentPlan    plan_,
                   Size         count_,
                   cudaStream_t stream_,
@@ -408,7 +409,7 @@ namespace core {
           stream(stream_),
           name(name_),
           debug_sync(debug_sync_),
-          grid(static_cast<unsigned int>(count + plan.items_per_tile - 1) / plan.items_per_tile),
+          grid(static_cast<unsigned int>((count + plan.items_per_tile - 1) / plan.items_per_tile)),
           vshmem(NULL),
           has_shmem((size_t)core::get_max_shared_memory_per_block() >= (size_t)plan.shared_memory_size),
           shmem_size(has_shmem ? plan.shared_memory_size : 0)
@@ -417,7 +418,7 @@ namespace core {
     }
 
     template <class Size>
-    CUB_RUNTIME_FUNCTION
+    THRUST_RUNTIME_FUNCTION
     AgentLauncher(AgentPlan    plan_,
                   Size         count_,
                   cudaStream_t stream_,
@@ -429,15 +430,15 @@ namespace core {
           stream(stream_),
           name(name_),
           debug_sync(debug_sync_),
-          grid(static_cast<unsigned int>(count + plan.items_per_tile - 1) / plan.items_per_tile),
+          grid(static_cast<unsigned int>((count + plan.items_per_tile - 1) / plan.items_per_tile)),
           vshmem(vshmem),
           has_shmem((size_t)core::get_max_shared_memory_per_block() >= (size_t)plan.shared_memory_size),
           shmem_size(has_shmem ? plan.shared_memory_size : 0)
     {
       assert(count > 0);
     }
-    
-    CUB_RUNTIME_FUNCTION
+
+    THRUST_RUNTIME_FUNCTION
     AgentLauncher(AgentPlan    plan_,
                   cudaStream_t stream_,
                   char const*  name_,
@@ -455,7 +456,7 @@ namespace core {
       assert(plan.grid_size > 0);
     }
 
-    CUB_RUNTIME_FUNCTION
+    THRUST_RUNTIME_FUNCTION
     AgentLauncher(AgentPlan    plan_,
                   cudaStream_t stream_,
                   char*        vshmem,
@@ -499,30 +500,34 @@ namespace core {
       return get_agent_plan<Agent>(sm_arch<0>::type::ver);
     }
 #endif
-    
-    CUB_RUNTIME_FUNCTION
+
+    THRUST_RUNTIME_FUNCTION
     typename core::get_plan<Agent>::type static get_plan(cudaStream_t , void* d_ptr = 0)
     {
       THRUST_UNUSED_VAR(d_ptr);
       core::cuda_optional<int> ptx_version = core::get_ptx_version();
       return get_agent_plan<Agent>(ptx_version);
     }
-    
+
     THRUST_RUNTIME_FUNCTION
     typename core::get_plan<Agent>::type static get_plan()
     {
       return get_agent_plan<Agent>(lowest_supported_sm_arch::ver);
     }
 
-    CUB_RUNTIME_FUNCTION void sync() const
+    THRUST_RUNTIME_FUNCTION void sync() const
     {
       if (debug_sync)
       {
-#ifdef __CUDA_ARCH__
-        cudaDeviceSynchronize();
-#else
-        cudaStreamSynchronize(stream);
-#endif
+        if (THRUST_IS_DEVICE_CODE) {
+          #if THRUST_INCLUDE_DEVICE_CODE
+            cudaDeviceSynchronize();
+          #endif
+        } else {
+          #if THRUST_INCLUDE_HOST_CODE
+            cudaStreamSynchronize(stream);
+          #endif
+        }
       }
     }
 
@@ -543,7 +548,7 @@ namespace core {
     }
 
 
-    
+
     template<class K>
     THRUST_RUNTIME_FUNCTION
     void print_info(K k) const
@@ -705,11 +710,11 @@ namespace core {
 
 #if 0
 
-    // If we are guaranteed to have enough shared memory 
+    // If we are guaranteed to have enough shared memory
     // don't compile other kernel which accepts pointer
     // and save on compilations
     template <class... Args>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch_impl(thrust::detail::true_type, Args... args) const
     {
       assert(has_shmem && vshmem == NULL);
@@ -717,17 +722,17 @@ namespace core {
       launcher::triple_chevron(grid, plan.block_threads, shmem_size, stream)
           .doit(_kernel_agent<Agent, Args...>, args...);
     }
-    
-    // If there is a risk of not having enough shared memory 
+
+    // If there is a risk of not having enough shared memory
     // we compile generic kernel instead.
     // This kernel is likely to be somewhat slower, but it can accomodate
     // both shared and virtualized shared memories.
     // Alternative option is to compile two kernels, one using shared and one
     // using virtualized shared memory. While this can be slightly faster if we
     // do actually have enough shared memory, the compilation time will double.
-    // 
+    //
     template <class... Args>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch_impl(thrust::detail::false_type, Args... args) const
     {
       assert((has_shmem && vshmem == NULL) || (!has_shmem && vshmem != NULL && shmem_size == 0));
@@ -737,7 +742,7 @@ namespace core {
     }
 
     template <class... Args>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch(Args... args) const
     {
 #if __THRUST__TEMPLATE_DEBUG
@@ -755,7 +760,7 @@ namespace core {
     }
 #else
     template <class _0>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch_impl(thrust::detail::false_type, _0 x0) const
     {
       assert((has_shmem && vshmem == NULL) || (!has_shmem && vshmem != NULL && shmem_size == 0));
@@ -765,7 +770,7 @@ namespace core {
           .doit(ptr, vshmem, x0);
     }
     template <class _0, class _1>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch_impl(thrust::detail::false_type, _0 x0, _1 x1) const
     {
       assert((has_shmem && vshmem == NULL) || (!has_shmem && vshmem != NULL && shmem_size == 0));
@@ -775,7 +780,7 @@ namespace core {
           .doit(ptr, vshmem, x0, x1);
     }
     template <class _0, class _1, class _2>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch_impl(thrust::detail::false_type, _0 x0, _1 x1, _2 x2) const
     {
       assert((has_shmem && vshmem == NULL) || (!has_shmem && vshmem != NULL && shmem_size == 0));
@@ -785,7 +790,7 @@ namespace core {
           .doit(ptr, vshmem, x0, x1, x2);
     }
     template <class _0, class _1, class _2, class _3>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch_impl(thrust::detail::false_type, _0 x0, _1 x1, _2 x2, _3 x3) const
     {
       assert((has_shmem && vshmem == NULL) || (!has_shmem && vshmem != NULL && shmem_size == 0));
@@ -795,7 +800,7 @@ namespace core {
           .doit(ptr, vshmem, x0, x1, x2, x3);
     }
     template <class _0, class _1, class _2, class _3, class _4>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch_impl(thrust::detail::false_type, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4) const
     {
       assert((has_shmem && vshmem == NULL) || (!has_shmem && vshmem != NULL && shmem_size == 0));
@@ -805,7 +810,7 @@ namespace core {
           .doit(ptr, vshmem, x0, x1, x2, x3, x4);
     }
     template <class _0, class _1, class _2, class _3, class _4, class _5>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch_impl(thrust::detail::false_type, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5) const
     {
       assert((has_shmem && vshmem == NULL) || (!has_shmem && vshmem != NULL && shmem_size == 0));
@@ -815,7 +820,7 @@ namespace core {
           .doit(ptr, vshmem, x0, x1, x2, x3, x4, x5);
     }
     template <class _0, class _1, class _2, class _3, class _4, class _5, class _6>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch_impl(thrust::detail::false_type, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6) const
     {
       assert((has_shmem && vshmem == NULL) || (!has_shmem && vshmem != NULL && shmem_size == 0));
@@ -825,7 +830,7 @@ namespace core {
           .doit(ptr, vshmem, x0, x1, x2, x3, x4, x5, x6);
     }
     template <class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch_impl(thrust::detail::false_type, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7) const
     {
       assert((has_shmem && vshmem == NULL) || (!has_shmem && vshmem != NULL && shmem_size == 0));
@@ -835,7 +840,7 @@ namespace core {
           .doit(ptr, vshmem, x0, x1, x2, x3, x4, x5, x6, x7);
     }
     template <class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch_impl(thrust::detail::false_type, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8) const
     {
       assert((has_shmem && vshmem == NULL) || (!has_shmem && vshmem != NULL && shmem_size == 0));
@@ -844,7 +849,7 @@ namespace core {
           .doit(ptr, vshmem, x0, x1, x2, x3, x4, x5, x6, x7, x8);
     }
     template <class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8, class _9>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch_impl(thrust::detail::false_type, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8, _9 x9) const
     {
       assert((has_shmem && vshmem == NULL) || (!has_shmem && vshmem != NULL && shmem_size == 0));
@@ -854,7 +859,7 @@ namespace core {
           .doit(ptr, vshmem, x0, x1, x2, x3, x4, x5, x6, x7, x8, x9);
     }
     template <class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8, class _9, class _xA>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch_impl(thrust::detail::false_type, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8, _9 x9,_xA xA) const
     {
       assert((has_shmem && vshmem == NULL) || (!has_shmem && vshmem != NULL && shmem_size == 0));
@@ -864,7 +869,7 @@ namespace core {
           .doit(ptr, vshmem, x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, xA);
     }
     template <class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8, class _9, class _xA, class _xB>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch_impl(thrust::detail::false_type, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8, _9 x9,_xA xA,_xB xB) const
     {
       assert((has_shmem && vshmem == NULL) || (!has_shmem && vshmem != NULL && shmem_size == 0));
@@ -874,7 +879,7 @@ namespace core {
           .doit(ptr, vshmem, x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, xA, xB);
     }
     template <class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8, class _9, class _xA, class _xB, class _xC>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch_impl(thrust::detail::false_type, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8, _9 x9,_xA xA,_xB xB,_xC xC) const
     {
       assert((has_shmem && vshmem == NULL) || (!has_shmem && vshmem != NULL && shmem_size == 0));
@@ -884,7 +889,7 @@ namespace core {
           .doit(ptr, vshmem, x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, xA, xB, xC);
     }
     template <class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8, class _9, class _xA, class _xB, class _xC, class _xD>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch_impl(thrust::detail::false_type, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8, _9 x9,_xA xA,_xB xB,_xC xC,_xD xD) const
     {
       assert((has_shmem && vshmem == NULL) || (!has_shmem && vshmem != NULL && shmem_size == 0));
@@ -894,7 +899,7 @@ namespace core {
           .doit(ptr, vshmem, x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, xA, xB, xC, xD);
     }
     template <class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8, class _9, class _xA, class _xB, class _xC, class _xD, class _xE>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch_impl(thrust::detail::false_type, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8, _9 x9,_xA xA,_xB xB,_xC xC,_xD xD,_xE xE) const
     {
       assert((has_shmem && vshmem == NULL) || (!has_shmem && vshmem != NULL && shmem_size == 0));
@@ -909,7 +914,7 @@ namespace core {
     ////////////////////////////////////////////////////////
 
     template <class _0>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch_impl(thrust::detail::true_type, _0 x0) const
     {
       assert(has_shmem && vshmem == NULL);
@@ -919,7 +924,7 @@ namespace core {
           .doit(ptr, x0);
     }
     template <class _0, class _1>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch_impl(thrust::detail::true_type, _0 x0, _1 x1) const
     {
       assert(has_shmem && vshmem == NULL);
@@ -929,7 +934,7 @@ namespace core {
           .doit(ptr, x0, x1);
     }
     template <class _0, class _1, class _2>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch_impl(thrust::detail::true_type, _0 x0, _1 x1, _2 x2) const
     {
       assert(has_shmem && vshmem == NULL);
@@ -939,7 +944,7 @@ namespace core {
           .doit(ptr, x0, x1, x2);
     }
     template <class _0, class _1, class _2, class _3>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch_impl(thrust::detail::true_type, _0 x0, _1 x1, _2 x2, _3 x3) const
     {
       assert(has_shmem && vshmem == NULL);
@@ -949,7 +954,7 @@ namespace core {
           .doit(ptr, x0, x1, x2, x3);
     }
     template <class _0, class _1, class _2, class _3, class _4>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch_impl(thrust::detail::true_type, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4) const
     {
       assert(has_shmem && vshmem == NULL);
@@ -959,7 +964,7 @@ namespace core {
           .doit(ptr, x0, x1, x2, x3, x4);
     }
     template <class _0, class _1, class _2, class _3, class _4, class _5>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch_impl(thrust::detail::true_type, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5) const
     {
       assert(has_shmem && vshmem == NULL);
@@ -969,7 +974,7 @@ namespace core {
           .doit(ptr, x0, x1, x2, x3, x4, x5);
     }
     template <class _0, class _1, class _2, class _3, class _4, class _5, class _6>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch_impl(thrust::detail::true_type, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6) const
     {
       assert(has_shmem && vshmem == NULL);
@@ -979,7 +984,7 @@ namespace core {
           .doit(ptr, x0, x1, x2, x3, x4, x5, x6);
     }
     template <class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch_impl(thrust::detail::true_type, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7) const
     {
       assert(has_shmem && vshmem == NULL);
@@ -989,7 +994,7 @@ namespace core {
           .doit(ptr, x0, x1, x2, x3, x4, x5, x6, x7);
     }
     template <class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch_impl(thrust::detail::true_type, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8) const
     {
       assert(has_shmem && vshmem == NULL);
@@ -999,7 +1004,7 @@ namespace core {
           .doit(ptr, x0, x1, x2, x3, x4, x5, x6, x7, x8);
     }
     template <class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8, class _9>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch_impl(thrust::detail::true_type, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8, _9 x9) const
     {
       assert(has_shmem && vshmem == NULL);
@@ -1009,7 +1014,7 @@ namespace core {
           .doit(ptr, x0, x1, x2, x3, x4, x5, x6, x7, x8, x9);
     }
     template <class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8, class _9, class _xA>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch_impl(thrust::detail::true_type, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8, _9 x9, _xA xA) const
     {
       assert(has_shmem && vshmem == NULL);
@@ -1019,7 +1024,7 @@ namespace core {
           .doit(ptr, x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, xA);
     }
     template <class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8, class _9, class _xA, class _xB>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch_impl(thrust::detail::true_type, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8, _9 x9, _xA xA, _xB xB) const
     {
       assert(has_shmem && vshmem == NULL);
@@ -1029,7 +1034,7 @@ namespace core {
           .doit(ptr, x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, xA, xB);
     }
     template <class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8, class _9, class _xA, class _xB, class _xC>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch_impl(thrust::detail::true_type, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8, _9 x9, _xA xA, _xB xB, _xC xC) const
     {
       assert(has_shmem && vshmem == NULL);
@@ -1039,7 +1044,7 @@ namespace core {
           .doit(ptr, x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, xA, xB, xC);
     }
     template <class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8, class _9, class _xA, class _xB, class _xC, class _xD>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch_impl(thrust::detail::true_type, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8, _9 x9, _xA xA, _xB xB, _xC xC, _xD xD) const
     {
       assert(has_shmem && vshmem == NULL);
@@ -1049,7 +1054,7 @@ namespace core {
           .doit(ptr, x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, xA, xB, xC, xD);
     }
     template <class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8, class _9, class _xA, class _xB, class _xC, class _xD, class _xE>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch_impl(thrust::detail::true_type, _0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8, _9 x9, _xA xA, _xB xB, _xC xC, _xD xD, _xE xE) const
     {
       assert(has_shmem && vshmem == NULL);
@@ -1062,107 +1067,107 @@ namespace core {
     ////////////////////////////////////////////////////////
     ////////////////////////////////////////////////////////
     ////////////////////////////////////////////////////////
-    
+
     template <class _0>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch(_0 x0) const
     {
       launch_impl(has_enough_shmem_t(), x0);
       sync();
     }
     template <class _0, class _1>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch(_0 x0, _1 x1) const
     {
       launch_impl(has_enough_shmem_t(), x0, x1);
       sync();
     }
     template <class _0, class _1, class _2>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch(_0 x0, _1 x1, _2 x2) const
     {
       launch_impl(has_enough_shmem_t(), x0, x1, x2);
       sync();
     }
     template <class _0, class _1, class _2, class _3>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch(_0 x0, _1 x1, _2 x2, _3 x3) const
     {
       launch_impl(has_enough_shmem_t(), x0, x1, x2, x3);
       sync();
     }
     template <class _0, class _1, class _2, class _3, class _4>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch(_0 x0, _1 x1, _2 x2, _3 x3, _4 x4) const
     {
       launch_impl(has_enough_shmem_t(), x0, x1, x2, x3, x4);
       sync();
     }
     template <class _0, class _1, class _2, class _3, class _4, class _5>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch(_0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5) const
     {
       launch_impl(has_enough_shmem_t(), x0, x1, x2, x3, x4, x5);
       sync();
     }
     template <class _0, class _1, class _2, class _3, class _4, class _5, class _6>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch(_0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6) const
     {
       launch_impl(has_enough_shmem_t(), x0, x1, x2, x3, x4, x5, x6);
       sync();
     }
     template <class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch(_0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7) const
     {
       launch_impl(has_enough_shmem_t(), x0, x1, x2, x3, x4, x5, x6, x7);
       sync();
     }
     template <class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch(_0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8) const
     {
       launch_impl(has_enough_shmem_t(), x0, x1, x2, x3, x4, x5, x6, x7, x8);
       sync();
     }
     template <class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8, class _9>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch(_0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8, _9 x9) const
     {
       launch_impl(has_enough_shmem_t(), x0, x1, x2, x3, x4, x5, x6, x7, x8, x9);
       sync();
     }
     template <class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8, class _9, class _xA>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch(_0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8, _9 x9, _xA xA) const
     {
       launch_impl(has_enough_shmem_t(), x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, xA);
       sync();
     }
     template <class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8, class _9, class _xA, class _xB>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch(_0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8, _9 x9, _xA xA, _xB xB) const
     {
       launch_impl(has_enough_shmem_t(), x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, xA, xB);
       sync();
     }
     template <class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8, class _9, class _xA, class _xB, class _xC>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch(_0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8, _9 x9, _xA xA, _xB xB, _xC xC) const
     {
       launch_impl(has_enough_shmem_t(), x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, xA, xB, xC);
       sync();
     }
     template <class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8, class _9, class _xA, class _xB, class _xC, class _xD>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch(_0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8, _9 x9, _xA xA, _xB xB, _xC xC, _xD xD) const
     {
       launch_impl(has_enough_shmem_t(), x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, xA, xB, xC, xD);
       sync();
     }
     template <class _0, class _1, class _2, class _3, class _4, class _5, class _6, class _7, class _8, class _9, class _xA, class _xB, class _xC, class _xD, class _xE>
-    void CUB_RUNTIME_FUNCTION
+    void THRUST_RUNTIME_FUNCTION
     launch(_0 x0, _1 x1, _2 x2, _3 x3, _4 x4, _5 x5, _6 x6, _7 x7, _8 x8, _9 x9, _xA xA, _xB xB, _xC xC, _xD xD, _xE xE) const
     {
       launch_impl(has_enough_shmem_t(), x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, xA, xB, xC, xD, xE);
@@ -1175,5 +1180,5 @@ namespace core {
 
 }    // namespace core
 }
-THRUST_END_NS
+} // end namespace thrust
 #endif
