@@ -47,7 +47,7 @@
  *         <tt>THRUST_VERSION / 100 % 1000</tt> is the minor version.
  *         <tt>THRUST_VERSION / 100000</tt> is the major version.
  */
-#define THRUST_VERSION 100905
+#define THRUST_VERSION 100910
 
 /*! \def THRUST_MAJOR_VERSION
  *  \brief The preprocessor macro \p THRUST_MAJOR_VERSION encodes the
@@ -71,10 +71,7 @@
  *  \brief The preprocessor macro \p THRUST_PATCH_NUMBER encodes the
  *         patch number of the Thrust library.
  */
-#define THRUST_PATCH_NUMBER 0
-
-
-// Declare these namespaces here for the purpose of Doxygenating them
+#define THRUST_PATCH_NUMBER 1
 
 /*! \namespace thrust
  *  \brief \p thrust is the top-level namespace which contains all Thrust
@@ -84,12 +81,3 @@ namespace thrust
 {
 
 }
-
-#ifndef THRUST_BEGIN_NS
-#define THRUST_BEGIN_NS namespace thrust {
-#endif
-
-#ifndef THRUST_END_NS
-#define THRUST_END_NS }
-#endif
-
